<?php defined('SYSPATH') or die('No direct access allowed.');

/**
 * This Model manages both the accounts that users use to login to the system, as well as the account where services are owned.
 *
 * @package    lnApp
 * @category   Models
 * @author     Deon George
 * @copyright  (c) 2014 Deon George
 * @license    http://dev.leenooks.net/license.html
 */
abstract class lnApp_Model_Account extends Model_Auth_UserDefault {
	// Relationships
	protected $_has_many = array(
		'email_log'=>array('far_key'=>'id'),
		'group'=>array('through'=>'account_group'),
	);

	protected $_belongs_to = array(
		'country'=>array(),
		'currency'=>array(),
		'language'=>array(),
	);

	protected $_display_filters = array(
		'active'=>array(
			array('StaticList_YesNo::get',array(':value',TRUE)),
		),
		'date_orig'=>array(
			array('Site::Date',array(':value')),
		),
		'date_last'=>array(
			array('Site::Date',array(':value')),
		),
	);

	protected $_form = array('id'=>'id','value'=>'name()');

	protected $_save_message = TRUE;

	/** REQUIRED ABSTRACT METHODS **/

	/** LOCAL METHODS **/

	/**
	 * Our account number format
	 */
	public function activate_code() {
		return md5(sprintf('%s-%s-%s-%s',$this->refnum(),$this->date_orig,$this->date_last,$this->email));
	}

	public function activated() {
		return $this->verified;
	}

	/**
	 * Get the groups that an account belongs to
	 */
	public function groups() {
		$result = array();

		foreach ($this->group->where_active()->find_all() as $go)
			foreach ($go->list_parentgrps(TRUE) as $cgo)
				if (empty($result[$cgo->id]))
					$result[$cgo->id] = $cgo;

		return $result;
	}

	public function log($message) {
		if (! class_exists('Model_Account_Log'))
			return TRUE;

		// Log a message for this account
		$alo = ORM::factory('Account_Log');
		$alo->account_id = $this->id;
		$alo->ip = Request::$client_ip;
		$alo->details = $message;
		$alo->save();

		return $alo->saved();
	}

	public function isAdmin() {
		return FALSE;
	}

	/**
	 * Return an account name
	 */
	public function name($variable=NULL) {
		return trim(sprintf('%s %s',$this->first_name,$this->last_name));
	}

	public function refnum($short=FALSE) {
		return ($short ? '' : sprintf('%02s-',Site::id())).sprintf('%04s',$this->id);
	}

	/**
	 * Return a token valid for this user
	 */
	public function token($token_expire,$module,$method,$uses) {
		return $this->id.':'.md5(sprintf('%s-%s',$this->refnum(),$this->date_last));
	}

	/**
	 * Search for accounts matching a term
	 */
	public function list_autocomplete($term,$index,$value,array $label,array $limit=array(),array $options=array()) {
		if (! isset($options['parentbypass'])) {
			$ao = Auth::instance()->get_user();

			$this->clear();
			$this->where_active();

			// Build our where clause
			// First Name, Last name
			if (preg_match('/\ /',$term)) {
				list($fn,$ln) = explode(' ',$term,2);

				$this->where_open()
					->where('first_name','like','%'.$fn.'%')
					->and_where('last_name','like','%'.$ln.'%')
					->where_close();

			} elseif (is_numeric($term)) {
				$this->where('id','like','%'.$term.'%');

			} elseif (preg_match('/\@/',$term)) {
				$this->where('email','like','%'.$term.'%');

			} else {
				$this->where_open()
					->or_where('first_name','like','%'.$term.'%')
					->or_where('last_name','like','%'.$term.'%')
					->or_where('email','like','%'.$term.'%')
					->where_close();
			}

			// Restrict results to authorised accounts
			// @todo
		}

		return parent::list_autocomplete($term,$index,$value,$label,$limit,$options);
	}
}
?>
